/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "ItemOutlineView.h"

@interface NSObject(ItemDataSource)
- (void) addItemToArrayItem:item;
- (void) removeItem:item;
@end
@interface NSObject(ItemViewDelegete)
- contextMenuForIdentifier:identifier;
@end
@implementation ItemOutlineView

- (NSMenu *)menuForEvent:(NSEvent *)e
{
	NSPoint p = [self convertPoint:[e locationInWindow] fromView:nil];
	int column = [self columnAtPoint:p];
	int row = [self selectedRow];
	NSArray* array = [self tableColumns];
	if(row >= 0 && column >= 0 && column < [array count]){
		NSTableColumn* tc = [array objectAtIndex:column];
		id item = [self itemAtRow:row];
		if([item respondsToSelector:@selector(contextMenuForIdentifier:)]){
			id menu = [item contextMenuForIdentifier:[tc identifier]];
			if(menu)
				return menu;
		}
	}
	return [super menuForEvent:e];
}

- (BOOL) performKeyEquivalent:(NSEvent*)e
{
	NSString* str = [e charactersIgnoringModifiers];
	unsigned mflags = [e modifierFlags];
	if((mflags & (NSControlKeyMask|NSAlternateKeyMask|NSCommandKeyMask)) == 0 && 
			[str characterAtIndex:0] == 127)
	{
		int row = [self selectedRow];
		if(row >= 0){
			[[self dataSource]performSelector:@selector(removeSelectedItem:) withObject:self];
			return YES;
		}
	}else
	if((mflags & (NSControlKeyMask|NSAlternateKeyMask|NSCommandKeyMask)) == 0 && 
			[str characterAtIndex:0] == '+')
	{
		int row = [self selectedRow];
		if(row >= 0){
			[[self dataSource]performSelector:@selector(insertNewItem:) withObject:self];
			return YES;
		}
	}
	return [super performKeyEquivalent:e];
}

- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)isLocal
{
	if(isLocal)
		return NSDragOperationEvery;
	else
		return NSDragOperationCopy | NSDragOperationGeneric;
}

@end
