/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "ResponseSimulator.h"
#import "IDLException.h"
#import "IDLDefinition_Private.h"
#import "IDLOctetStream.h"
#import "VXMessageHeader.h"

@implementation ResponseSimulator

static id _instance = nil;

- (NSString*) nibName
{
	return @"ResponseSimulator";
}

- init
{
	self = [super init];
	[NSBundle loadNibNamed:[self nibName] owner:self];
	return self;
}

+ createSimulator
{
	if(!_instance){
		_instance = [[self alloc]init];
		[[NSNotificationCenter defaultCenter]postNotificationName:SimulatorDidActivate object:_instance];
	}
	return _instance;
}

+ simulator
{
	return _instance;
}

- (void) dealloc
{
	[[NSNotificationCenter defaultCenter]removeObserver:self];
	_instance = nil;
	[super dealloc];
}

- (void) awakeFromNib
{
	[_view setFont:[NSFont userFixedPitchFontOfSize:11.0]];
}

- window
{
	return _window;
}

- (void) windowWillClose:(NSNotification*)note
{
	[[NSNotificationCenter defaultCenter]postNotificationName:SimulatorWillDeactivate object:self];
	[self autorelease];
}

- (NSMutableData*) response
{
	NSString* string = [_view string];
	NSData* source = [string dataUsingEncoding:NSUTF8StringEncoding];
	int l = [source length];
	const char* b = [source bytes];
	NSMutableData* data = [NSMutableData data];
	unsigned char c;
	unsigned char x;
	BOOL first = YES;
	while(l-- > 0){
		if(*b >= '0' && *b <='9')
			c = *b - '0';
		else if(*b >= 'a' && *b <= 'f')
			c = *b - 'a' + 10;
		else if(*b >= 'A' && *b <= 'F')
			c = *b - 'A' + 10;
		else{
			b += 1;
			continue;
		}
		if(first){
			x = (c << 4);
			first = NO;
		}else{
			x |= c;
			first = YES;
			[data appendBytes:&x length:1];
		}
		b += 1;
	}
	if(!first){
		NSLog(@"Response Simulator: odd number of hex digits");
		[data appendBytes:&x length:1];
	}
	return data;
}

static BOOL isHex(unichar c)
{
	return c >= '0' && c <= '9' || c >='a' && c<= 'f' || c >= 'A' && c <= 'F';
}

- (void) selectAtIndex:(int)index
{
	NSTextStorage* storage = [_view textStorage];
	NSString* string = [storage string];
	int length = [storage length], pos = -1, n;
	for(n=0;n<length;n++){
		if(isHex([string characterAtIndex:n]))
			pos += 1;
		if(pos == 2*index)
			break;
	}
	[storage beginEditing];
	[storage setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
				[NSColor grayColor],NSForegroundColorAttributeName,
				[NSFont userFixedPitchFontOfSize:11.0], NSFontAttributeName, nil]
			range:NSMakeRange(0, length)];
	[storage setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
				[NSColor blackColor],NSForegroundColorAttributeName,
				[NSFont userFixedPitchFontOfSize:11.0], NSFontAttributeName, nil]
			range:NSMakeRange(n, length-n)];
	[storage endEditing];
}

- (void) selectRange:(NSRange)range
{
	NSTextStorage* storage = [_view textStorage];
	NSString* string = [storage string];
	int length = [storage length], pos = -1, n0, n1;
	for(n0=0;n0<length;n0++){
		if(isHex([string characterAtIndex:n0]))
			pos += 1;
		if(pos == 2*range.location)
			break;
	}
	for(n1=n0;n1<length;n1++){
		if(pos == 2*(range.location + range.length))
			break;
		if(isHex([string characterAtIndex:n1]))
			pos += 1;
	}
	if(n0 <= length){
		if(n1 > length)
			n1 = length;
		[storage beginEditing];
		[storage setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
				[NSColor grayColor],NSForegroundColorAttributeName,
				[NSFont userFixedPitchFontOfSize:11.0], NSFontAttributeName, nil]
				range:NSMakeRange(0, length)];
		[storage setAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
				[NSColor blackColor],NSForegroundColorAttributeName,
				[NSFont userFixedPitchFontOfSize:11.0], NSFontAttributeName, nil]
				range:NSMakeRange(n0, n1-n0)];
		[storage endEditing];
	}
}

- (int) selectionStart
{
	return _selectionStart;
}

- (int) selectionLength
{
	return _selectionLength;
}

- (void) setSelectionStart:(int)pos
{
	_selectionStart = pos;
	[self selectRange:NSMakeRange(_selectionStart, _selectionLength)];
}

- (void) setSelectionLength:(int)length
{
	_selectionLength = length;
	[self selectRange:NSMakeRange(_selectionStart, _selectionLength)];
}

- (void)textViewDidChangeSelection:(NSNotification *)aNotification
{
	NSRange r = [_view selectedRange];
	NSRange r1;
	NSString* string = [_view string];
	int pos = -1, n, length = [string length];
	
	for(n=0;n<=MIN(r.location, length-1);n++){
		if(isHex([string characterAtIndex:n]))
			pos += 1;
	}
	r1.location = (pos+1)/2;
	for(n=r.location+1;n<r.location+r.length;n++){
		if(isHex([string characterAtIndex:n]))
			pos += 1;
	}
	r1.length = (pos+1)/2 - r1.location;
	[self willChangeValueForKey:@"selectionStart"];
	[self willChangeValueForKey:@"selectionLength"];
	_selectionStart = r1.location;
	_selectionLength = r1.length;
	[self didChangeValueForKey:@"selectionLength"];
	[self didChangeValueForKey:@"selectionStart"];
	[self selectRange:r1];
}

@end

@implementation VXMessage(ResponseSimulator)

- initWithResponseStream:(IDLOctetStream*)stream
{
	self = [super init];
	_stream = [stream retain];
	_header = [[VXMessageHeader alloc]initWithStream:_stream]; // sets byte order and GIOP version
	return self;
}
@end

@implementation IDLFunction(ResponseSimulator)
- (NSDictionary*) simulateInvokeWith:(NSDictionary*)args response:(NSMutableData*)data
{
	if(![self isOneWay] && data){
		NSMutableDictionary* dict = args? [NSMutableDictionary dictionaryWithDictionary:args] :
                            [NSMutableDictionary dictionary];
		IDLOctetStream* stream = [IDLOctetStream octetStreamWithData:data];
		VXMessage* response = [[VXMessage alloc]initWithResponseStream:stream];

		NSString* path = [NSString stringWithFormat:@"GIOP/ReplyHeader_%d_%d",
				[response majorVersion], [response minorVersion]];
		IDLDefinition* def = (IDLDefinition*)[[IDLDefinitionScope rootScope]definitionForNamePath:path];
		unsigned char replyStatus = GIOP_NoException;
		if(def){
			id replyHeader = nil;
			[def decodeObject:&replyHeader withCoder:[response stream]];
			replyStatus = [[replyHeader valueForKey:@"reply_status"]intValue];
		}else
			[NSException raise:CORBADefinitionException format:@"No definition for %@",path];

		if(replyStatus != GIOP_NoException){
			NSString* reason = @"unknown";
			switch(replyStatus){
				case GIOP_LocationForward:
					reason = @"LocationForward"; break;
				case GIOP_LocationForwardPerm:
					reason = @"LocationForwardPerm"; break;
				case GIOP_UserException:
					reason = @"UserException"; break;
				case GIOP_SystemException:
					reason = @"SystemException"; break;
				default:
					reason = [NSString stringWithFormat:@"unknown(%d)", replyStatus]; break;
			}
			[NSException raise:@"GIOPException" format:@"Response contains GIOP Exception '%@'", reason];
		}else{
			[response alignStreamBody];
			[self decodeObject:&dict withCoder:[response stream]];
		}
		return dict;
	}else
		return args;
}

@end

NSString* SimulatorDidActivate = @"SimulatorDidActivate";
NSString* SimulatorWillDeactivate = @"SimulatorWillDeactivate";
