/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Cocoa/Cocoa.h>
#import <ADORB/ADORB.h>

int main (int argc, const char *argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    NSString* interfacePath = [defaults stringForKey:@"i"]; // interface path
	NSString* file = [defaults stringForKey:@"file"];
	int rc = 0;
	@try{
		IDLInterface* object = interfacePath? [[ORB defaultORB]interfaceNamed:interfacePath] : nil;
		if(object){
			NSString* header = [object generateHeader];
			NSFileHandle* handle = nil;
			if(file)
				handle = [NSFileHandle fileHandleForWritingAtPath:file];
			else
				handle = [NSFileHandle fileHandleWithStandardOutput];
			[handle writeData:[header dataUsingEncoding:NSUTF8StringEncoding]];
		}else if(interfacePath){
			NSLog(@"Invalid interface path '%@'", interfacePath);
			rc = 1;
		}else{
			NSLog(@"Format: %@ -ORBidl idl-file -i interface-path [-file path]", [[NSProcessInfo processInfo]processName]);
			rc = 1;
		}
	}@catch(NSException *e){
		NSLog(@"Exception %@, %@", [e name], [e reason]);
		rc = 1;
	}
    [pool release];
    exit(rc);
    return 0;
}

