/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinition_Private.h"
#import "IDLDefinitionScope_Private.h"
#import "IDLArray.h"
#import "IDLArrayIdentifier.h"
#import "IDLTypedef.h"
#import "IDLStruct.h"
#import "IDLInterface_Private.h"
#import "IDLEnum.h"

@implementation IDLDefinitionScope

extern void initScope(IDLDefinitionScope* scope);
static IDLDefinitionScope* _rootScope = nil;
static NSMutableArray* _instances = nil;

static NSString* _currentPrefix = nil;

extern NSString* currentParserFile();

NSString* CORBADefinitionException = @"CORBADefinitionException";
NSString* IDLInvalidArgumentException = @"IDLInvalidArgumentException";

+ (NSArray*) _instances
{
    return _instances;
}

+ (void) _setCurrentPrefix:(NSString*) string
{
	int l = [string length];
	if(l > 1){
		int c0 = [string characterAtIndex:0];
		int c1 = [string characterAtIndex:l-1];
		if(c0 == '\"' || c1 == '\"'){
			if(c0 == '\"' && c1 == '\"')
				string = [string substringWithRange:NSMakeRange(1, l-2)];
			else
				NSLog(@"*** Parser Error: invalid #pragma prefix %@", string);
		}
	}
    [_currentPrefix release];
    _currentPrefix = [string retain];
}

+ (NSString*) currentPrefix
{
    return _currentPrefix;
}

+ (void) _registerDefaults
{
}

+ (void) _resetDefaults:(NSNotification*)notification
{
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    NSString* debug = [defaults stringForKey:@"debug"];
    if([debug rangeOfString:@"M"].length > 0)
        _instances = [[NSMutableArray alloc]init];
}

+ (NSZone*) zone
{
    return [IDLDefinition zone];
}

+ (void) pushScope:(IDLDefinitionScope*)scope
{
    NSMutableDictionary* dict = [[NSThread currentThread]threadDictionary];
    NSMutableArray* scopeStack = [dict objectForKey:@"ORB_ScopeStack"];
    IDLDefinitionScope* currentScope = [dict objectForKey:@"ORB_CurrentScope"];
    if(!scopeStack)
        [dict setObject:scopeStack = [NSMutableArray array] forKey:@"ORB_ScopeStack"];
        
    if(currentScope)
        [scopeStack addObject:currentScope];
    [dict setObject:scope forKey:@"ORB_CurrentScope"];
}

+ (IDLDefinitionScope*)popScope
{
    NSMutableDictionary* dict = [[NSThread currentThread]threadDictionary];
    NSMutableArray* scopeStack = [dict objectForKey:@"ORB_ScopeStack"];
    IDLDefinitionScope* currentScope = nil;
   
     if([scopeStack count] > 0){
        currentScope = [scopeStack lastObject];
        [dict setObject:currentScope forKey:@"ORB_CurrentScope"];
        [scopeStack removeLastObject];
    }else
        [dict removeObjectForKey:@"ORB_CurrentScope"];
    return currentScope;
}

+ (IDLDefinitionScope*)currentScope
{
    NSMutableDictionary* dict = [[NSThread currentThread]threadDictionary];
    return [dict objectForKey:@"ORB_CurrentScope"];
}

+ (void) clearScopeStack
{
    NSMutableDictionary* dict = [[NSThread currentThread]threadDictionary];
	[dict removeObjectForKey:@"ORB_ScopeStack"];
	[dict removeObjectForKey:@"ORB_CurrentScope"];
}

- (void) set
{
    NSMutableDictionary* dict = [[NSThread currentThread]threadDictionary];
    [dict setObject:self forKey:@"ORB_CurrentScope"];
}

- init
{
    [super init];
    _definitionDict = [[NSMutableDictionary alloc]init];
    //_dict = [[NSMutableDictionary alloc]init];
    _prefix = [_currentPrefix retain];
    _versions = [[NSMutableDictionary alloc]init];
    [_instances addObject:self];
    return self;
}

- (void) dealloc
{
    [_definitionDict release];
    [(NSObject*)_parentScope release];
    [_scopeName release];
    //[_dict release];
    [_namePath release];
    [_prefix release];
    [_versions release];
    [super dealloc];
}

+ (IDLDefinitionScope*) definitionScope
{
    IDLDefinitionScope* scope = [[[self allocWithZone:[self zone]]init]autorelease];
    return scope;
}

+ (IDLDefinitionScope*) definitionScopeWithParentScope:(id<IDLDefinitionScope>)pscope
{
    IDLDefinitionScope* scope = [[[self allocWithZone:[self zone]]init]autorelease];
    [scope setParentScope:pscope];
    return scope;
}

- (id <IDLDefinition>) _definitionNamed:(NSString*)name
{
    return [_definitionDict objectForKey:name];
}

- (id <IDLDefinition>) localDefinitionNamed:(NSString*)name
{
    return [self _definitionNamed:name];
}

+ _findScopeWithName:(NSString*)name
{
// _findScopeWithName: - looking for scope locally only
    return [[self currentScope]_definitionNamed:name];
}

+ (id <IDLDefinitionScope>)definitionScopeWithName:(NSString*)name
{
    id <IDLScopePrivate>scope = [self _findScopeWithName:name];
    if(scope == nil){
        scope = [self definitionScope];
        [scope setName:name];
    }
    return scope;
}

- (void) addDefinition:(id <IDLDefinition>)def withName:(NSString*)name
{
    if(def && name)
        [_definitionDict setObject:def forKey:name];
}

- addDefinition:(id <IDLDefinition>)def withIdentifierArray:(NSArray*)identArray
{
    id identifier;
    NSEnumerator* en = [identArray objectEnumerator];
	NSMutableArray* array = [NSMutableArray array];
    while(identifier = [en nextObject]){
        NSString* name;
        BOOL isArray = [identifier isKindOfClass:[IDLArrayIdentifier class]];
        if(isArray)
            name = [identifier name];
        else
            name = identifier;
                    
        if([self _definitionNamed:name]){
            NSLog(@"Redefining name '%@'\n",name); 
        }else{
            IDLTypedef* tdef;
            if(isArray){
                IDLArray* obj = [IDLArray arrayWithType:(IDLDefinition*)def identifier:identifier];
                tdef = [IDLTypedef typedefWithType:obj name:name];
            }else
                tdef = [IDLTypedef typedefWithType:(IDLDefinition*)def name:name];
            [self addDefinition:tdef withName:name];
			[array addObject:tdef];
        }
    }
    return [array count] > 0? array : nil;
}

- (id <IDLDefinition>)definitionNamed:(NSString*)name
{
    id <IDLDefinition> obj = [self _definitionNamed:name];
    if(!obj){
        if(_parentScope)
            obj = [_parentScope definitionNamed:name];
        else
        if(self != _rootScope)
            obj = [_rootScope definitionNamed:name];
    }
    return obj;
}

- (NSDictionary*)_definitionDictionary
{
    return _definitionDict;
}

- (id <IDLDefinitionScope>) parentScope
{
    return _parentScope;
}

- (IDLDefinitionScope*) rootScope
{
    id scope = [self parentScope];
    if(scope)
        return [scope rootScope];
    else
        return self;
}

- (void) setParentScope:(id<IDLDefinitionScope>)scope
{
    if(_parentScope == nil)
        _parentScope = [(NSObject*)scope retain];
    else
    if(_parentScope != scope)
        NSLog(@"Trying to re-set parent scope for %@ to %@\n",
            [self description],
            [(NSObject*)scope description]);
}

- (NSString*)name
{
    return _scopeName;
}
- (void) setName:(NSString*)name
{
    [_scopeName release];
    _scopeName = [name copy];
}

- initRootScope
{
    [self init];
    initScope(self);
    return self;
}

+ (IDLDefinitionScope*) rootScope
{
    if(!_rootScope){
        _rootScope = [[IDLDefinitionScope allocWithZone:[self zone]]init];
        initScope(_rootScope);
		//[_rootScope setName:@"SystemRoot"];
    }
    return _rootScope;
}

- (NSEnumerator*) objectEnumerator
{
    return [_definitionDict objectEnumerator];
}
- (NSEnumerator*) keyEnumerator
{
    return [_definitionDict keyEnumerator];
}
- (NSArray*)allKeys;
{
    return [_definitionDict allKeys];
}

- (void) defineValue:(NSString*)value forKey:(NSString*)key
{
	if(!_defineDict)
		_defineDict = [[NSMutableDictionary alloc]init];
	[_defineDict setObject:value forKey:key];
}

- (void) undefineValueForKey:(NSString*)key
{
	[_defineDict removeObjectForKey:key];
}

- (void) undefineKey:(NSString*)key
{
	[self undefineValueForKey:key];
}

- (NSString*) definedValueForKey:(NSString*)key
{
	return [_defineDict objectForKey:key];
}

- (void) awakeFromParse
{
    NSEnumerator* en = [self objectEnumerator];
    IDLDefinition* obj;

    while(obj = [en nextObject]){
        if([obj isKindOfClass:[IDLDefinition class]])
            [obj awakeFromParse];
    }
}

- (NSString*) description
{
    return [NSString stringWithFormat:@"<%@ 0x%08x> %@",
        NSStringFromClass([self class]), self, _scopeName?_scopeName:((self == _rootScope)?@"SystemRoot" : @"RootModule")];
}

- (void) applyVersions
{
    NSEnumerator* en = [_versions keyEnumerator];
    NSString* name;
    while(name = [en nextObject]){
        IDLDefinition* def = (IDLDefinition*)[self definitionForNamePath:name];
        if([def respondsToSelector:@selector(setVersion:)])
            [def setVersion:[_versions objectForKey:name]];
    }
}

- (void) setNamePath:(NSString*)path
{
    NSEnumerator* en = [_definitionDict objectEnumerator];
    IDLDefinition* obj;
    if(path != _namePath){
        [_namePath release];
        _namePath = [path copy];
    }
    path = _namePath?[NSString stringWithFormat:@"%@/%@", _namePath,[self name]] : [self name];
    while(obj = [en nextObject])
        [obj setNamePath:path];
}

- (NSString*)version
{
    return @"1.0";
}

- (void) setVersion:(NSString*)value
{
    
}

- (NSString*)prefix
{
    return _prefix;
}

- (NSString*)_path
{
    if(_namePath)
        return [NSString stringWithFormat:@"%@/%@",_namePath,[self name]];
    else
        return [self name];
}
- (NSString*) typeId
{
    NSString* prefix = [self prefix];
    if([prefix length] > 0)
        return [NSString stringWithFormat:@"IDL:%@/%@:%@",
                    [self prefix],[self _path],[self version]];
    else
        return [NSString stringWithFormat:@"IDL:%@:%@",
                    [self _path],[self version]];
}

// This method is too flexible
- (id <IDLDefinition>) definitionForNamePath:(NSString*)path
{
    if([path length] > 0){
        NSRange r = [path rangeOfString:@"/"];
        if(r.length > 0){
            if(r.location > 0){
                IDLDefinition* obj = (IDLDefinition*)[self definitionNamed:[path substringToIndex:r.location]];
                if([obj respondsToSelector:_cmd])
                    return (IDLDefinition*)[(id<IDLDefinitionScope>)obj definitionForNamePath:[path substringFromIndex:r.location+r.length]];
                else
                    return nil;
            }else
                return [[self rootScope]definitionForNamePath:[path substringFromIndex:r.location + r.length]];
        }else
            return [self definitionNamed:path];
    }
    return nil;
}

- valueForKey:(NSString*)key
{
	if([key characterAtIndex:0] == '@')
		return [super valueForKey:[key substringFromIndex:1]];
	else
		return [self definitionNamed:key];
}

static NSString* _reverse(NSString* str)
{
    if(str){
        NSEnumerator* en = [[str componentsSeparatedByString:@"."]reverseObjectEnumerator];
        NSString* name;
        NSMutableArray* array = [NSMutableArray array];
        while(name = [en nextObject])
            [array addObject:name];
        return [array componentsJoinedByString:@"."];
    }else
        return nil;
}

// Format: IDL:<prefix><path>:<version>
// <path>: /module/def
// <prefix>: xorb.com
// 'typeId' always contains an absolute path and this method is always sent to the root scope
// The root scope always has prefix = nil
// 'typeId' is defined ambigiously - first name in the 'path' may be prefix or module
- (id <IDLDefinition>) definitionForTypeId:(NSString*)typeId
{
    IDLDefinition* obj = nil;
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    @try{
		if([typeId hasPrefix:@"IDL:"]){
			NSString* path;
			NSString* path2 = nil;
			NSRange r = {4, [typeId length] - 4};
			NSRange r1 = [typeId rangeOfString:@":" options:0 range:r];
			if(r1.length > 0)
				r.length = r1.location - r.location;
			path = [typeId substringWithRange:r];
			r = [path rangeOfString:@"/"];
			if(r.length > 0){
				if(r.location > 0){ // starts with name or with prefix
					NSString* name0 = [path substringToIndex:r.location];
					NSRange r2 = [name0 rangeOfString:@"."];
					if(r2.length > 0) // if has '.' it is the prefix
						path = [path substringFromIndex:r.location+r.length];
					else // may be prefix or may be not ?
						path2 = [path substringFromIndex:r.location+r.length];
				}else // starts with '/'
					path = [path substringFromIndex:r.location+r.length];
			}
			obj = (IDLDefinition*)[self definitionForNamePath:path];
			if(!obj && path2)
				obj = (IDLDefinition*)[self definitionForNamePath:path2];
		}else
		if([typeId hasPrefix:@"RMI:"]){
			NSRange r = {4, [typeId length] - 4};
			NSRange r1 = [typeId rangeOfString:@":" options:0 range:r];
			NSString* prefixPath = [_reverse([self prefix]) stringByAppendingString:@"."];
			NSString* path;
			if(r1.length > 0)
				r.length = r1.location - r.location;
			path = [typeId substringWithRange:r];
			if(prefixPath && [path hasPrefix:prefixPath])
				path = [path substringFromIndex:[prefixPath length]];
			path = [[path componentsSeparatedByString:@"."]componentsJoinedByString:@"/"];
			obj = (IDLDefinition*)[self definitionForNamePath:path];
		}
    }@catch(NSException *localException){
		NSLog(@"[%@ %@]: %@, %@\n",
			NSStringFromClass([self class]), NSStringFromSelector(_cmd),
			[localException name], [localException reason]);
    }
    [pool release];
    if(obj)
        return obj;
    else
    if(self != _rootScope) // Look in system definitions
        return (IDLDefinition*)[_rootScope definitionForTypeId:typeId];
    else
        return nil;
}

- (NSNumber*) enumValueForName:(NSString*)name andKey:(NSString*)key 
{
    if(name && key){
        IDLEnum* def = (IDLEnum*)[self definitionForNamePath:name];
        if(def)
            return [def enumValueForKey:key];
        else
            [NSException raise:IDLInvalidArgumentException 
                format:@"Cannot locate enum definition for '%@'",
                name];
    }else
        [NSException raise:IDLInvalidArgumentException 
            format:@"NULL argument in [%@ %@]",
            NSStringFromClass([self class]),NSStringFromSelector(_cmd)];
    
    return nil;
}

- (long) kind
{
    return 0;
}

- (void) _setVersion:(NSString*)version forDefinitionNamed:(NSString*)name
{
    if(version && name){
        if(_parentScope){
            name = [NSString stringWithFormat:@"%@/%@",_scopeName, name];
            [(IDLDefinitionScope*)_parentScope _setVersion:version forDefinitionNamed:name];
        }else
            [_versions setObject:version forKey:name];
    }
}

- (IDLDefinition*) referenceType
{
	return nil;
}

- (BOOL) isDerivedFromIDLType:(IDLDefinition*)type
{
	return (id)type == self;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_definitionDict forKey:@"definitions"];
		[coder encodeConditionalObject:_parentScope forKey:@"parentScope"];
		[coder encodeObject:_scopeName forKey:@"scopeName"];
		[coder encodeObject:_namePath forKey:@"namePath"];
		[coder encodeObject:_prefix forKey:@"prefix"];
		[coder encodeObject:_defineDict forKey:@"define"];
	}else{
		[coder encodeObject:_definitionDict];
		[coder encodeConditionalObject:_parentScope];
		[coder encodeObject:_scopeName];
		[coder encodeObject:_namePath];
		[coder encodeObject:_prefix];
		[coder encodeObject:_defineDict];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_definitionDict = [[coder decodeObjectForKey:@"definitions"]retain];
		_parentScope = [[coder decodeObjectForKey:@"parentScope"]retain];
		_scopeName = [[coder decodeObjectForKey:@"scopeName"]retain];
		_namePath = [[coder decodeObjectForKey:@"namePath"]retain];
		_prefix = [[coder decodeObjectForKey:@"prefix"]retain];
		_defineDict = [[coder decodeObjectForKey:@"define"]retain];
	}else{
		_definitionDict = [[coder decodeObject]retain];
		_parentScope = [[coder decodeObject]retain];
		_scopeName = [[coder decodeObject]retain];
		_namePath = [[coder decodeObject]retain];
		_prefix = [[coder decodeObject]retain];
		_defineDict = [[coder decodeObject]retain];
	}
    _versions = [[NSMutableDictionary alloc]init];
	return self;
}

@end

@implementation IDLDefinitionScope(Pragma)

extern int currentParserLine();

+ (void) performCommand:command with:(NSArray*)values
{
    SEL selector;
    if([command isKindOfClass:[NSArray class]]){
        NSMutableArray* array = [NSMutableArray arrayWithArray:command];
        [array removeObjectAtIndex:0];
        [array addObjectsFromArray:values];
        values = array;
        command = [command objectAtIndex:0];
    }
    selector = NSSelectorFromString([NSString stringWithFormat:@"perform%@:",[command capitalizedString]]);
    if(selector && [self respondsToSelector:selector])
        [self performSelector:selector withObject:values];
    else
        NSLog(@"%@:%d: '%@ %@' is not mapped. Ignoring\n",
                currentParserFile(), currentParserLine(), command,[values description]);
}

+ (void) _pragmaPrefix:(NSArray*)args
{
    if([args count] >= 1)
        [self _setCurrentPrefix:[args objectAtIndex:0]];
}

+ (void) _pragmaClass:(NSArray*)args
{
    if([args count] >= 2){
        NSString* name = [args objectAtIndex:0];
        NSString* className = [args objectAtIndex:1];
        IDLDefinition* obj = (IDLDefinition*)[[IDLDefinitionScope currentScope]definitionNamed:name];
        if([obj respondsToSelector:@selector(setObjectClassName:)])
            [(IDLStruct*)obj setObjectClassName:className];
    }
}

+ (void) _pragmaMap:(NSArray*)args
{
    [self _pragmaClass:args];
}

+ (void) _pragmaVersion:(NSArray*)args
{
    if([args count] >= 2){
        NSString* name = [args objectAtIndex:0];
        NSString* version = [args objectAtIndex:1];
        [[IDLDefinitionScope currentScope] _setVersion:version forDefinitionNamed:name];
    }
}

+ (void) _pragmaId:(NSArray*) args
{
    if([args count] >= 2){
        NSString* name = [args objectAtIndex:0];
        NSString* typeId = [args objectAtIndex:1];
        IDLDefinition* def = (IDLDefinition*)[[IDLDefinitionScope currentScope]_definitionNamed:name];
        [def setTypeId:typeId];
    }
}

// Sets pragma that specifies objc selector for a ORB method in IDL 
// #pragma selector method-name objc-selector
// For example for the ORB method:  void bind_context(in n, in naming_context)
// we can specify selector as: #pragma selector bind_context bindContextWithName:namingContext:
//
// The #pragma selector should appear after method definition in the same scope
//
+ (void) _pragmaSelector:(NSArray*)args
{
	if([args count] >= 2){
		NSString *name = [args objectAtIndex:0];
		NSString *selName = [args objectAtIndex:1];
        IDLDefinition *obj = (IDLDefinition*)[[IDLDefinitionScope currentScope]_definitionNamed:name];
		if([obj respondsToSelector:@selector(setSelectorName:)])
			[(IDLFunction*)obj setSelectorName:selName];
		else
			NSLog(@"%@:%d: invalid #pragma selector, method name '%@' is not found",
				  currentParserFile(), currentParserLine(), name);
	}else
		NSLog(@"%@:%d: #pragma selector requires two parameters: method name and selector",
			  currentParserFile(), currentParserLine());
}

+ (void) performPragma:(NSArray*)values
{
    if([values count] > 0){
        NSString* command = [values objectAtIndex:0];
        NSMutableArray* array = [NSMutableArray arrayWithArray:values];
        SEL selector = NSSelectorFromString([NSString stringWithFormat:@"_pragma%@:",
				[command capitalizedString]]);
        if(selector && [self respondsToSelector:selector]){
            [array removeObjectAtIndex:0];
            [self performSelector:selector withObject:array];
        }else
            NSLog(@"%@:%d: pragma command '%@' is not mapped. Ignoring.\n",
                currentParserFile(),currentParserLine(), command);
    }else
        NSLog(@"%@:%d: No command is specified in pragma.\n",
            currentParserFile(), currentParserLine());
}


@end
