/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLDefinition.h"
#import "CORBACoding.h"
#import "IDLTCKind.h"

@class IDLNamedDefinition, IDLEnum, IDLFunction, IDLInterface, IDLOrderedDictionary, IDLSequence, IDLPrimitive, IDLStruct;

@interface IDLDefinition(Private)

- (BOOL) isOneWay;
- convertValue:aValue;
- coerceValue:aValue;

- sequenceClass;
- arrayClass;

- (BOOL) sortMatrixItems;
- (int) sortIfGreaterThan;
+ (BOOL) sortMatrixItems;
+ (int) sortIfGreaterThan;
+ (void) setSortMatrixItems:(BOOL)flag;
+ (void) setSortIfGreaterThan:(int)n;

- (NSString*) valueDescriptionOf:aValue;
- (void) setVersion:(NSString*)aString;

- (void) setTypeId:(NSString*)str;
- (void) setNamePath:(NSString*)path;

@end

@interface IDLDefinition(CORBACoding) <CORBACoding>

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n;
- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr;
- (void) setValue:value ofType:(const char*)type forPointer:(void*)ptr;
- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n;
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv;
- returnValueFromInvocation:(NSInvocation*)inv;
- (void) encodeWithCORBACoder:(id<CORBACoder>)coder;
+ definitionFromCoder:(id<CORBADecoder>)coder;
+ _definitionFromCoder:(id<CORBADecoder>)coder;
+ _definitionFromCoder:(id<CORBADecoder>)coder tag:(long)tag;
+ (NSString*) stringForKind:(long)kind;
- (BOOL) willSendReference;

// internal classes that encode/decode themselves should call these methods
+ objectForValue:(void*)data ofKind:(TCKind)kind;
+ (void) setValue:(void*)data fromObject:object ofKind:(TCKind)kind;

@end

@interface IDLDefinition(IDLParser)
- (void) awakeFromParse;
@end

@interface NSDictionary(IDLParser)
- (void) awakeFromParse;
@end

extern void ORBLogUndefinedKeyError(IDLDefinition *def, id object, NSString* key);


