/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLException.h"
#import "IDLDefinition_Private.h"

@implementation IDLException

static NSMutableDictionary* _aliases = nil;
NSString* IDLExceptionTypeKey = @"_exceptionType";
NSString* IDLExceptionIDKey = @"exception_id";
NSString* IDLExceptionKey = @"$IDLException";
NSString* IDLMinorCodeKey = @"minor";
NSString* IDLCompletionStatusKey = @"completed";

+ (void) initialize
{
    if([IDLException class] == self){
        NSString* path = [[NSBundle bundleForClass:self] pathForResource:@"Exceptions" ofType:@"plist"];
        if(path)
            _aliases = [[NSMutableDictionary alloc]initWithContentsOfFile:path];
        
    }
}

+ exceptionWithName:(NSString*)aName members:(NSArray*)members
{
    return [super structWithName:aName members:members];
}

+ exceptionWithStruct:(IDLStruct*)obj andName:(NSString*)name
{
    return [self exceptionWithName:name members:[obj members]];
}

- (NSString*) description
{
    return [NSString stringWithFormat:@"<%@ 0x%x> exception %@",
        NSStringFromClass([self class]), self, [self name]];
}


- (void) encodeObject:obj withCoder:(id <CORBACoder>)inv
{
    // Encode the name first.
    // Note that 'decode' does not demarshal the name.
    [inv encodeString:[self typeId]];
    [super encodeObject:obj withCoder:inv];
}

- (long) kind
{
    return tk_except;
}

+ (NSDictionary*) systemInfoForException:(NSException*)exc
{
    NSString* name = [exc name];
    return [_aliases objectForKey:name];
}

+ (NSDictionary*) systemInfoWithName:(NSString*)excName
{
    return [_aliases objectForKey:excName];
}

+ (NSDictionary*) systemExceptionInfo:(NSString*)corbaName minor:(long)minor completed:(CORBA_CompletionStatus)cs
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
        [NSNumber numberWithLong:minor],IDLMinorCodeKey,
        [NSNumber numberWithLong:cs],IDLCompletionStatusKey,
        [NSNumber numberWithLong:GIOP_SystemException],IDLExceptionTypeKey,
        [NSString stringWithFormat:@"IDL:omg.org/CORBA/%@:1.0", corbaName],IDLExceptionIDKey,
        nil];
}

@end
