/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLNamedDefinition.h"
#import "IDLDefinitionScope.h"
#import <expr.tab.h>

@implementation IDLNamedDefinition

+ (NSZone*)zone
{
    return [IDLDefinition zone];
}

- initWithType:(IDLDefinition*)type name:(NSString*)anIdent
{
    self = [super init];
    _ident = [anIdent copy];
    _def = [type retain];
    return self;
}

+ definitionFor:(NSString*)anIdent ofType:(IDLDefinition*)type
{
    return [[[self allocWithZone:[self zone]]initWithType:type name:anIdent]autorelease];
}

- (void) dealloc
{
    [_ident release];
    [_def release];
    [super dealloc];
}
	
- (NSString*)ident
{
    return _ident;
}

- (IDLDefinition*)definition
{
    return _def;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeConditionalObject:_def forKey:@"def"];
		[coder encodeObject:_ident forKey:@"ident"];
	}else{
		[coder encodeConditionalObject:_def];
		[coder encodeObject:_ident];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_def = [[coder decodeObjectForKey:@"def"]retain];
		_ident = [[coder decodeObjectForKey:@"ident"]retain];
	}else{
		_def = [[coder decodeObject]retain];
		_ident = [[coder decodeObject]retain];
	}
	return self;
}

@end

