/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLNative.h"
#import "IDLEncapsulation.h"

@interface NSObject(IDLNative)
+ sharedInstance;
@end

@implementation IDLNative

- initWithName:(NSString*)name
{
    self = [super init];
    _name = [name retain];
    return self;
}

+ (IDLNative*) nativeType:(NSString*)name
{
    NSString* className = [NSString stringWithFormat:@"IDL%@",name];
    id class = NSClassFromString(className);
    if(class && [class respondsToSelector:@selector(sharedInstance)])
        return [class sharedInstance];
    else
        return [[[self alloc]initWithName:name]autorelease];
}

- (void) dealloc
{
    [_name release];
    [_className release];
    [super dealloc];
}

- (void) setClassName:(NSString*)name
{
    _className = [name retain];
}

- (NSString*)className
{
    return _className;
}

- (NSString*) name
{
    return _name;
}


-(NSString*) description
{
    return [[super description] stringByAppendingFormat:@"native %@",_name];
}

- (long) kind
{
    return tk_native;
}

- (void) _encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [coder encodeString:[self typeId]];
    [coder encodeString:[self name]];
}
- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
    [coder encodeLong:[self kind]];
    [stream setParentStream:coder location:[coder location]];
    [self _encodeWithCORBACoder:stream];
    [coder encodeData:[stream data]];
}

+ _definitionFromCoder:(IDLEncapsulation*)coder
{
    NSString* typeId;
    NSString* name;
    [coder decodeString:&typeId];
    [coder decodeString:&name];
    return [self nativeType:name];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_className forKey:@"className"];
	}else{
		[coder encodeObject:_name];
		[coder encodeObject:_className];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
	}else{
		_name = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
	}
	return self;
}

@end
