/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLReference.h"
#import "IDLDefinitionScope.h"
#import "IDLDefinition_Private.h"
#import "IDLArray.h"
#import "IDLSequence.h"

@implementation IDLReference

- initWithName:(NSString*)name inScope:(IDLDefinitionScope*)scope
{
    self = [super init];
    _scope = scope; // Don't retain
    _name = [name retain];
    return self;
}

- (void) dealloc
{
    [_name release];
    [super dealloc];

}

+ reference:(NSString*)name inScope:(IDLDefinitionScope*)scope
{
    return [[[self allocWithZone:[IDLDefinition zone]]initWithName:name inScope:scope]autorelease];
}

- (NSMethodSignature*) methodSignatureForSelector:(SEL)selector
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        return [def methodSignatureForSelector:selector];
    else{
        NSLog(@"Runtime error: cannot resolve definition name '%@'\n", _name);
        [NSException raise:CORBADefinitionException
            format:@"cannot resolve definition name '%@'", _name];
        return nil;
    }
        
}

- (void) forwardInvocation:(NSInvocation*)invocation
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        [invocation invokeWithTarget:def];
    else
        [self doesNotRecognizeSelector:[invocation selector]];
}

- (id <IDLDefinition>)referenceType
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    return [def referenceType];
}

- valueForKey:(NSString*)key
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
	return [def valueForKey:key];
}

- (NSString*) name
{
    return _name;
}

- (void) setNamePath:(NSString*)path
{
}

- sequenceClass
{
    return [IDLSequence class];
}

- arrayClass
{
    return [IDLArray class];
}

- (NSString*)description
{
    IDLDefinition* def = (IDLDefinition*)[_scope definitionNamed:_name];
    if(def)
        return [NSString stringWithFormat:@"@%@",def];
    else
        return [NSString stringWithFormat:@"<%@:0x%x %@>",
            NSStringFromClass([self class]), self, _name];
}

- (id)replacementObjectForKeyedArchiver:(NSKeyedArchiver *)archiver
{
	return [_scope definitionNamed:_name];
}
- (id)replacementObjectForCoder:(NSCoder *)aCoder
{
	return [_scope definitionNamed:_name];
}
@end
