/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLSequence.h"
#import "IDLEncapsulation.h"
#import "IDLLink.h"

@implementation IDLSequence(CORBACoding)

- (void) encodeObject:object withCoder:(id <CORBACoder>)inv
{
    long length = [object count];
    NSEnumerator* en = [object objectEnumerator];
    id item;
	[inv encodeULong:length];
    while(item = [en nextObject])
        [type encodeObject:item withCoder:inv];
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)inv
{
    [inv decodeSequence:obj usingObjectCoder:type];
}

- (const char*) objCType
{
    return @encode(id);
}
- (unsigned) argLength
{
    return sizeof(id);
}
- (unsigned) returnValueLength
{
    return sizeof(id);
}

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    // Sequence is mapped to NSArray
    id array = nil;
    [inv getArgument:&array atIndex:n];
    return array;
}

- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
    id array = *((id*)ptr);
    return array;
}

- (void) setValue:value ofType:(const char*)argType forPointer:(void*)ptr
{
    *((id*)ptr) = value;
}
- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    id _val = value;
    [inv setArgument:&_val atIndex:n];
}
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    id _val = value;
    [inv setReturnValue:&_val];
}
- returnValueFromInvocation:(NSInvocation*)inv
{
    id _val = nil;
    [inv getReturnValue:&_val];
    return _val;
}

- (void) _encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [type encodeWithCORBACoder:coder];
    [coder encodeULong:_frameCount];
}

- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
    [coder encodeLong:[self kind]];
    [stream setParentStream:coder location:[coder location]];
    // Don't use indirection for sequence
    //[stream registerLocation:-2*sizeof(long) forDefinition:self];
    [self _encodeWithCORBACoder:stream];
    [coder encodeData:[stream data]];
}

+ _definitionFromCoder:(IDLEncapsulation*)coder
{
    unsigned long cnt = 0;
    IDLDefinition* def;
    IDLLink* link = [IDLLink linkWithDefinition:nil];
    
    [coder registerDefinition:(IDLDefinition*)link forLocation:-2*sizeof(long)];

    def = [IDLDefinition definitionFromCoder:coder];
    [coder decodeULong:&cnt];
    [link setLink:[IDLSequence sequenceWithType:def count:cnt]];
    
    return link;
}

@end

