/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLEncapsulation.h"

@implementation IDLEncapsulation

- init
{
    self = [super initWithStreamData:[NSMutableData data] offset:0];
    _flags._isOutStream = YES;
    [self useByteOrderFromFirstOctet];
    return self;
}

- initWithStreamData:(NSMutableData*)data
{
    self = [super initWithStreamData:data offset:0];
    [self useByteOrderFromFirstOctet];
    return self;
}

+ encapsulation
{
    return [[[self alloc]init]autorelease];
}

+ encapsulationWithData:(NSMutableData*)data
{
    return [[[self alloc]initWithStreamData:data]autorelease];
}

- (void) dealloc
{
    [_ref release];
    [super dealloc];
}

- (void) setParentStream:stream location:(long)location
{
    if([stream isKindOfClass:[IDLEncapsulation class]]){// topmost encapsulation has no parent
        _parentStream = stream; // don't retain
        _location = location + sizeof(long); // encapsulation is encoded as sequence<octet>
    }
	if(stream){ // Inherit string & wstring encodings
		[self setStringEncoding:[stream stringEncoding]];
		[self setWStringEncoding:[stream wstringEncoding]];
	}
}

- (void) _addToLocation:(long*)loc
{
    *loc += _location;
    [_parentStream _addToLocation:loc];
}

- (long) _currentLocation
{
    long loc = [_data length];
    [self _addToLocation:&loc];
    return loc;
}

- (void) encodeIndirection:(long)location
{
    long offset = -1;
    [self _alignTo:sizeof(long)];
    [_data appendBytes:&offset length:sizeof(long)]; // indirection kind
    offset = location - [self _currentLocation];
    [_data appendBytes:&offset length:sizeof(long)];
}

// Pass location in the current stream, which may be negative
- (IDLDefinition*) definitionWithLocation:(long)location
{
    if(_parentStream == nil){
        NSString* key = [NSString stringWithFormat:@"l=%d",location];
        IDLDefinition* def = (IDLDefinition*)[_ref objectForKey:key];
        if(def)
            return def;
        [NSException raise:CORBAMarshalException format:@"Invalid indirection"];
        return nil;
    }else
        return [_parentStream definitionWithLocation:location + _location];
}

// returns location in topmost encapsulation
- (long) locationForDefinition:(IDLDefinition*)def
{
    if(_parentStream == nil){
        NSString* key = [NSString stringWithFormat:@"0x%x",def];
        return [[_ref objectForKey:key]longValue];
    }else
        return [_parentStream locationForDefinition:def];
}

- (void) registerDefinition:(IDLDefinition*)def forLocation:(long)loc
{
    if(_parentStream == nil){
        if(!_ref)
            _ref = [[NSMutableDictionary alloc]init];
        if(def){
            NSString* key = [NSString stringWithFormat:@"l=%d",loc];
            [_ref setObject:def forKey:key];
        }
    }else
        [_parentStream registerDefinition:def forLocation:loc + _location];
}

- (void) registerDefinition:(IDLDefinition*)def
{
    [self registerDefinition:def forLocation:-2*sizeof(long)];
}

- (void) registerLocation:(long)location forDefinition:(IDLDefinition*)def
{
    if(_parentStream == nil){
        NSString* key = [NSString stringWithFormat:@"0x%x",def];
        if(!_ref)
            _ref = [[NSMutableDictionary alloc]init];
        [_ref setObject:[NSNumber numberWithLong:location] forKey:key];
    }else
        [_parentStream registerLocation:location + _location forDefinition:def];
}

- (VXMessage*)message
{
	return [_parentStream message];
}

- (void) setMessage:(VXMessage*)message
{
}

@end
